@@ -61,14 +61,13 @@
 #include <stdio.h>
 #include <string.h>
 
-static GtkTargetEntry drag_types [] = {
+static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
 	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
-	{ NAUTILUS_ICON_DND_TEXT_TYPE, 0, NAUTILUS_ICON_DND_TEXT }
 };
 
-static GtkTargetEntry drop_types [] = {
+static const GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
 	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
@@ -375,12 +374,16 @@ get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context,
 	GList *target;
 	GtkTargetList *list;
 	
-	if (drop_types_list == NULL)
+	if (drop_types_list == NULL) {
 		drop_types_list = gtk_target_list_new (drop_types,
 						       G_N_ELEMENTS (drop_types) - 1);
-	if (drop_types_list_root == NULL)
+		gtk_target_list_add_text_targets (drop_types_list, NAUTILUS_ICON_DND_TEXT);
+	}
+	if (drop_types_list_root == NULL) {
 		drop_types_list_root = gtk_target_list_new (drop_types,
 							    G_N_ELEMENTS (drop_types));
+		gtk_target_list_add_text_targets (drop_types_list_root, NAUTILUS_ICON_DND_TEXT);
+	}
 
 	if (nautilus_icon_container_get_is_desktop (NAUTILUS_ICON_CONTAINER (widget))) {
 		list = drop_types_list_root;
@@ -643,6 +646,20 @@ receive_dropped_uri_list (NautilusIconContainer *container, const char *uri_list
 				 x, y);
 }
 
+/* handle dropped text */
+static void
+receive_dropped_text (NautilusIconContainer *container, const char *text, GdkDragAction action, int x, int y)
+{	
+	if (text == NULL) {
+		return;
+	}
+	
+	g_signal_emit_by_name (container, "handle_text",
+			       text,
+			       action,
+			       x, y);
+}
+
 static int
 auto_scroll_timeout_callback (gpointer data)
 {
@@ -751,21 +768,21 @@ confirm_switch_to_manual_layout (NautilusIconContainer *container)
 	 */
 	if (nautilus_icon_container_has_stored_icon_positions (container)) {
 		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = _("Do you want to switch to manual layout and leave this item where you dropped it? "
+			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it? "
 			"This will clobber the stored manual layout.");
-			detail = _("This folder uses automatic layout.");
+			detail = no_translate("This folder uses automatic layout.");
 		} else {
-			message = _("Do you want to switch to manual layout and leave these items where you dropped them? "
+			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them? "
 			"This will clobber the stored manual layout.");
-			detail = _("This folder uses automatic layout.");
+			detail = no_translate("This folder uses automatic layout.");
 		}
 	} else {
 		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = _("Do you want to switch to manual layout and leave this item where you dropped it?");
-			detail = _("This folder uses automatic layout.");
+			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it?");
+			detail = no_translate("This folder uses automatic layout.");
 		} else {
-			message = _("Do you want to switch to manual layout and leave these items where you dropped them?");
-			detail = _("This folder uses automatic layout.");
+			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them?");
+			detail = no_translate("This folder uses automatic layout.");
 
 		}
 	}
@@ -1131,6 +1148,7 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		break;
 
 	case NAUTILUS_ICON_DND_TEXT:
+		*action = GDK_ACTION_COPY;
 		break;
 	}
 }
@@ -1239,34 +1257,22 @@ drag_leave_callback (GtkWidget *widget,
 	nautilus_icon_container_free_drag_data(NAUTILUS_ICON_CONTAINER (widget));
 }
 
-void
-nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
-			      GdkDragAction actions,
-			      int button,
-			      GdkEventMotion *event,
-			      int                    start_x,
-			      int                    start_y)
+static void
+drag_begin_callback (GtkWidget      *widget,
+		     GdkDragContext *context,
+		     gpointer        data)
 {
-	NautilusIconDndInfo *dnd_info;
-	EelCanvas *canvas;
-	GdkDragContext *context;
+	NautilusIconContainer *container;
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
-	int x_offset, y_offset;
 	double x1, y1, x2, y2, winx, winy;
-	
-	g_return_if_fail (NAUTILUS_IS_ICON_CONTAINER (container));
-	g_return_if_fail (event != NULL);
+	int x_offset, y_offset;
+	int start_x, start_y;
 
-	dnd_info = container->details->dnd_info;
-	g_return_if_fail (dnd_info != NULL);
-	
-	/* Notice that the event is in bin_window coordinates, because of
-           the way the canvas handles events.
-	*/
-	canvas = EEL_CANVAS (container);
-	dnd_info->drag_info.start_x = start_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (canvas)));
-	dnd_info->drag_info.start_y = start_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (canvas)));	
+	container = NAUTILUS_ICON_CONTAINER (widget);
+
+	start_x = container->details->dnd_info->drag_info.start_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
+	start_y = container->details->dnd_info->drag_info.start_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));
 
         /* create a pixmap and mask to drag with */
         pixmap = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item, &mask);
@@ -1278,24 +1284,46 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
         /* compute the image's offset */
 	eel_canvas_item_get_bounds (EEL_CANVAS_ITEM (container->details->drag_icon->item),
 				    &x1, &y1, &x2, &y2);
-	eel_canvas_world_to_window (canvas, x1, y1,  &winx, &winy);
+	eel_canvas_world_to_window (EEL_CANVAS (container), 
+				    x1, y1,  &winx, &winy);
         x_offset = start_x - winx;
         y_offset = start_y - winy;
-        
+
+	gtk_drag_set_icon_pixmap (context,
+				  gtk_widget_get_colormap (GTK_WIDGET (container)),
+				  pixmap, mask,
+				  x_offset, y_offset);
+}
+
+void
+nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
+			      GdkDragAction actions,
+			      int button,
+			      GdkEventMotion *event,
+			      int                    start_x,
+			      int                    start_y)
+{
+	NautilusIconDndInfo *dnd_info;
+	GdkDragContext *context;
+	
+	g_return_if_fail (NAUTILUS_IS_ICON_CONTAINER (container));
+	g_return_if_fail (event != NULL);
+
+	dnd_info = container->details->dnd_info;
+	g_return_if_fail (dnd_info != NULL);
+	
+	/* Notice that the event is in bin_window coordinates, because of
+           the way the canvas handles events.
+	*/
+	dnd_info->drag_info.start_x = start_x - gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
+	dnd_info->drag_info.start_y = start_y - gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));	
+
 	/* start the drag */
 	context = gtk_drag_begin (GTK_WIDGET (container),
 				  dnd_info->drag_info.target_list,
 				  actions,
 				  button,
 				  (GdkEvent *) event);
-
-	if (context) {
-		/* set the icon for dragging */
-		gtk_drag_set_icon_pixmap (context,
-					  gtk_widget_get_colormap (GTK_WIDGET (container)),
-					  pixmap, mask,
-					  x_offset, y_offset);
-	}
 }
 
 static gboolean
@@ -1321,7 +1349,6 @@ drag_highlight_expose (GtkWidget      *widget,
 			    widget->style->black_gc,
 			    FALSE,
 			    x, y, width - 1, height - 1);
-
 	
 	return FALSE;
 }
@@ -1342,33 +1369,43 @@ dnd_highlight_queue_redraw (GtkWidget *widget)
 	width = widget->allocation.width;
 	height = widget->allocation.height;
 
+	/* we don't know how wide the shadow is exactly,
+	 * so we expose a 10-pixel wide border
+	 */
 	gtk_widget_queue_draw_area (widget,
 				    0, 0,
-				    width, 1);
+				    width, 10);
 	gtk_widget_queue_draw_area (widget,
 				    0, 0,
-				    1, height);
+				    10, height);
 	gtk_widget_queue_draw_area (widget,
-				    0, height - 1,
-				    width, 1);
+				    0, height - 10,
+				    width, 10);
 	gtk_widget_queue_draw_area (widget,
-				    width - 1, 0,
-				    1, height);
+				    width - 10, 0,
+				    10, height);
 }
 
 static void
 start_dnd_highlight (GtkWidget *widget)
 {
 	NautilusIconDndInfo *dnd_info;
+	GtkWidget *toplevel;
 	
 	dnd_info = NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info;
 
+	toplevel = gtk_widget_get_toplevel (widget);
+	if (toplevel != NULL &&
+	    g_object_get_data (G_OBJECT (toplevel), "is_desktop_window")) {
+		return;
+	}
+
 	if (!dnd_info->highlighted) {
 		dnd_info->highlighted = TRUE;
 		g_signal_connect_after (widget, "expose_event",
 					G_CALLBACK (drag_highlight_expose),
 					NULL);
-		gtk_widget_queue_draw (widget);
+		dnd_highlight_queue_redraw (widget);
 	}
 }
 
@@ -1380,11 +1417,11 @@ stop_dnd_highlight (GtkWidget *widget)
 	dnd_info = NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info;
 
 	if (dnd_info->highlighted) {
-		dnd_info->highlighted = FALSE;
 		g_signal_handlers_disconnect_by_func (widget,
 						      drag_highlight_expose,
 						      NULL);
-		gtk_widget_queue_draw (widget);
+		dnd_highlight_queue_redraw (widget);
+		dnd_info->highlighted = FALSE;
 	}
 }
 
@@ -1542,6 +1579,7 @@ drag_data_received_callback (GtkWidget *widget,
 {
     	NautilusDragInfo *drag_info;
 	EelBackground *background;
+	char *tmp;
 	gboolean success;
 
 	drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
@@ -1557,6 +1595,7 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_BGIMAGE:	
 	case NAUTILUS_ICON_DND_KEYWORD:
 	case NAUTILUS_ICON_DND_URI_LIST:
+	case NAUTILUS_ICON_DND_TEXT:
 	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 		/* Save the data so we can do the actual work on drop. */
 		if (drag_info->selection_data != NULL) {
@@ -1619,6 +1658,14 @@ drag_data_received_callback (GtkWidget *widget,
 				 (char *) data->data, context->action, x, y);
 			success = TRUE;
 			break;
+		case NAUTILUS_ICON_DND_TEXT:
+			tmp = gtk_selection_data_get_text (data);
+			receive_dropped_text
+				(NAUTILUS_ICON_CONTAINER (widget),
+				 (char *) tmp, context->action, x, y);
+			success = TRUE;
+			g_free (tmp);
+			break;
 		case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 			background = eel_get_widget_background (widget);
 			if (background != NULL) {
@@ -1661,6 +1708,7 @@ void
 nautilus_icon_dnd_init (NautilusIconContainer *container,
 			GdkBitmap *stipple)
 {
+	GtkTargetList *targets;
 	int n_elements;
 	
 	g_return_if_fail (container != NULL);
@@ -1669,7 +1717,7 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 
 	container->details->dnd_info = g_new0 (NautilusIconDndInfo, 1);
 	nautilus_drag_init (&container->details->dnd_info->drag_info,
-		drag_types, G_N_ELEMENTS (drag_types));
+		drag_types, G_N_ELEMENTS (drag_types), TRUE);
 
 	/* Set up the widget as a drag destination.
 	 * (But not a source, as drags starting from this widget will be
@@ -1685,7 +1733,13 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 			   drop_types, n_elements,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK);
 
+	targets = gtk_drag_dest_get_target_list (GTK_WIDGET (container));
+	gtk_target_list_add_text_targets (targets, NAUTILUS_ICON_DND_TEXT);
+
+	
 	/* Messages for outgoing drag. */
+	g_signal_connect (container, "drag_begin", 
+			  G_CALLBACK (drag_begin_callback), NULL);
 	g_signal_connect (container, "drag_data_get",
 			  G_CALLBACK (drag_data_get_callback), NULL);
 	g_signal_connect (container, "drag_end",
