@@ -37,6 +37,7 @@
 #include "nautilus-main.h"
 #include "nautilus-signaller.h"
 #include "nautilus-location-bar.h"
+#include "nautilus-pathbar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
 #include <eel/eel-accessibility.h>
@@ -81,6 +82,7 @@
 #include <math.h>
 #include <sys/time.h>
 
+
 /* FIXME bugzilla.gnome.org 41243: 
  * We should use inheritance instead of these special cases
  * for the desktop window.
@@ -93,6 +95,8 @@
 
 #define MAX_TITLE_LENGTH 180
 
+#define MENU_PATH_BOOKMARKS_PLACEHOLDER			"/MenuBar/Other Menus/Bookmarks/Bookmarks Placeholder"
+
 enum {
 	ARG_0,
 	ARG_APP_ID,
@@ -108,6 +112,12 @@ static void side_panel_image_changed_callback        (NautilusSidebar          *
 static void navigation_bar_location_changed_callback (GtkWidget                *widget,
 						      const char               *uri,
 						      NautilusNavigationWindow *window);
+static void navigation_bar_cancel_callback           (GtkWidget                *widget,
+						      NautilusNavigationWindow *window);
+static void path_bar_location_changed_callback       (GtkWidget                *widget,
+						      const char               *uri,
+						      NautilusNavigationWindow *window);
+static void always_use_location_entry_changed        (gpointer                  callback_data);
 
 
 GNOME_CLASS_BOILERPLATE (NautilusNavigationWindow, nautilus_navigation_window,
@@ -121,6 +131,7 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	GtkWidget *location_bar;
 	GtkWidget *view_as_menu_vbox;
 	GtkToolItem *item;
+	GtkWidget *hbox;
 	
 	window->details = g_new0 (NautilusNavigationWindowDetails, 1);
 
@@ -131,24 +142,24 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	window->details->content_paned = nautilus_horizontal_splitter_new ();
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  window->details->content_paned,
-			  /* X direction */       /* Y direction */
-			  0, 1,                   3, 4,
-			  GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
-			  0,                      0);
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               3, 4,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
+			  0,                                  0);
 	gtk_widget_show (window->details->content_paned);
 
 	nautilus_navigation_window_initialize_actions (window);
 	nautilus_navigation_window_initialize_menus (window);
-
+	
 	ui_manager = nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window));
 	toolbar = gtk_ui_manager_get_widget (ui_manager, "/Toolbar");
 	window->details->toolbar = toolbar;
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  toolbar,
-			  /* X direction */       /* Y direction */
-			  0, 1,                   1, 2,
-			  GTK_EXPAND | GTK_FILL,  0,
-			  0,                      0);
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               1, 2,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0,
+			  0,                                  0);
 	gtk_widget_show (toolbar);
 
 	nautilus_navigation_window_initialize_toolbars (window);
@@ -162,22 +173,38 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	/* set up location bar */
 	location_bar = gtk_toolbar_new ();
 	window->details->location_bar = location_bar;
-	
-	window->navigation_bar = nautilus_location_bar_new (window);
-	gtk_widget_show (GTK_WIDGET (window->navigation_bar));
-
-	g_signal_connect_object (window->navigation_bar, "location_changed",
-				 G_CALLBACK (navigation_bar_location_changed_callback), window, 0);
 
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox);
 
 	item = gtk_tool_item_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (item), GNOME_PAD_SMALL);
 	gtk_widget_show (GTK_WIDGET (item));
 	gtk_tool_item_set_expand (item, TRUE);
-	gtk_container_add (GTK_CONTAINER (item),  window->navigation_bar);
+	gtk_container_add (GTK_CONTAINER (item),  hbox);
 	gtk_toolbar_insert (GTK_TOOLBAR (location_bar),
 			    item, -1);
 
+	window->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
+ 	gtk_widget_show (window->path_bar);
+	
+	g_signal_connect_object (window->path_bar, "path_clicked",
+				 G_CALLBACK (path_bar_location_changed_callback), window, 0);
+	
+	gtk_box_pack_start (GTK_BOX (hbox),
+			    window->path_bar,
+			    TRUE, TRUE, 0);
+
+	window->navigation_bar = nautilus_location_bar_new (window);
+	g_signal_connect_object (window->navigation_bar, "location_changed",
+				 G_CALLBACK (navigation_bar_location_changed_callback), window, 0);
+	g_signal_connect_object (window->navigation_bar, "cancel",
+				 G_CALLBACK (navigation_bar_cancel_callback), window, 0);
+
+	gtk_box_pack_start (GTK_BOX (hbox),
+			    window->navigation_bar,
+			    TRUE, TRUE, 0);
+
 	/* Option menu for content view types; it's empty here, filled in when a uri is set.
 	 * Pack it into vbox so it doesn't grow vertically when location bar does. 
 	 */
@@ -223,12 +250,90 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  location_bar,
-			  /* X direction */       /* Y direction */
-			  0, 1,                   2, 3,
-			  GTK_EXPAND | GTK_FILL,  0,
-			  0,                      0);
-	
-	
+			  /* X direction */                    /* Y direction */
+			  0, 1,                                2, 3,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,  0,
+			  0,                                   0);
+
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY,
+						  always_use_location_entry_changed,
+						  window, G_OBJECT (window));
+}
+
+static void
+always_use_location_entry_changed (gpointer callback_data)
+{
+	NautilusNavigationWindow *window;
+
+	window = NAUTILUS_NAVIGATION_WINDOW (callback_data);
+
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+		nautilus_navigation_window_hide_path_bar (window);
+	} else {
+		nautilus_navigation_window_show_path_bar (window);
+	}
+}
+
+static int
+bookmark_list_get_uri_index (GList *list,
+			     const char *uri)
+{
+	NautilusBookmark *bookmark;
+	GList *l;
+	char *tmp;
+	int i;
+
+	g_return_val_if_fail (uri != NULL, -1);
+
+	for (i = 0, l = list; l != NULL; i++, l = l->next) {
+		bookmark = NAUTILUS_BOOKMARK (l->data);
+
+		tmp = nautilus_bookmark_get_uri (bookmark);
+		if (strcmp (tmp, uri) == 0) {
+			g_free (tmp);
+			return i;
+		}
+		g_free (tmp);
+	}
+
+	return -1;
+}
+
+static void
+path_bar_location_changed_callback (GtkWidget *widget,
+				    const char *uri,
+				    NautilusNavigationWindow *window)
+{
+	int i;
+
+	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
+
+	/* check whether we already visited the target location */
+	i = bookmark_list_get_uri_index (window->back_list, uri);
+	if (i >= 0) {
+		nautilus_navigation_window_back_or_forward (window, TRUE, i);
+	} else {
+		nautilus_window_go_to (NAUTILUS_WINDOW (window), uri);
+	}
+}
+
+static void
+hide_temporary_bars (NautilusNavigationWindow *window)
+{
+	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
+
+	if (window->details->temporary_location_bar) {
+		if (nautilus_navigation_window_location_bar_showing (window)) {
+			nautilus_navigation_window_hide_location_bar (window, FALSE);
+		}
+		window->details->temporary_location_bar = FALSE;
+	}
+	if (window->details->temporary_navigation_bar) {
+		if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+			nautilus_navigation_window_show_path_bar (window);
+		}
+		window->details->temporary_navigation_bar = FALSE;
+	}
 }
 
 static void
@@ -236,18 +341,17 @@ navigation_bar_location_changed_callback (GtkWidget *widget,
 					  const char *uri,
 					  NautilusNavigationWindow *window)
 {
-	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
-
-	if (window->details->temporary_navigation_bar) {
-		if (nautilus_navigation_window_location_bar_showing (window)) {
-			nautilus_navigation_window_hide_location_bar (window, FALSE);
-		}
-		window->details->temporary_navigation_bar = FALSE;
-	}
-
+	hide_temporary_bars (window);
 	nautilus_window_go_to (NAUTILUS_WINDOW (window), uri);
 }
 
+static void
+navigation_bar_cancel_callback (GtkWidget *widget,
+				NautilusNavigationWindow *window)
+{
+	hide_temporary_bars (window);
+}
+
 static void
 side_pane_close_requested_callback (GtkWidget *widget,
 				    gpointer user_data)
@@ -416,10 +520,11 @@ nautilus_navigation_window_destroy (GtkObject *object)
 
 	window->view_as_option_menu = NULL;
 	window->navigation_bar = NULL;
+	window->path_bar = NULL;
 	window->zoom_control = NULL;
 
 	window->details->content_paned = NULL;
-
+	
 	if (window->details->tooltips) {
 		g_object_unref (G_OBJECT (window->details->tooltips));
 		window->details->tooltips = NULL;
@@ -435,7 +540,6 @@ nautilus_navigation_window_finalize (GObject *object)
 	
 	window = NAUTILUS_NAVIGATION_WINDOW (object);
 
-	nautilus_navigation_window_remove_bookmarks_menu_callback (window);
 	nautilus_navigation_window_remove_go_menu_callback (window);
 	nautilus_navigation_window_clear_back_list (window);
 	nautilus_navigation_window_clear_forward_list (window);
@@ -455,6 +559,7 @@ nautilus_navigation_window_add_sidebar_panel (NautilusNavigationWindow *window,
 {
 	const char *sidebar_id;
 	char *label;
+	char *tooltip;
 	char *default_id;
 	GdkPixbuf *icon;
 
@@ -464,9 +569,12 @@ nautilus_navigation_window_add_sidebar_panel (NautilusNavigationWindow *window,
 	g_return_if_fail (g_list_find (window->sidebar_panels, sidebar_panel) == NULL);	
 
 	label = nautilus_sidebar_get_tab_label (sidebar_panel);
+	tooltip = nautilus_sidebar_get_tab_tooltip (sidebar_panel);
 	nautilus_side_pane_add_panel (window->sidebar, 
 				      GTK_WIDGET (sidebar_panel), 
-				      label);
+				      label,
+				      tooltip);
+	g_free (tooltip);
 	g_free (label);
 
 	icon = nautilus_sidebar_get_tab_icon (sidebar_panel);
@@ -690,6 +798,12 @@ real_set_title (NautilusWindow *window, const char *title)
 	g_free (full_title);
 }
 
+static char *
+real_get_icon_name (NautilusWindow *window)
+{
+	return g_strdup ("file-manager");
+}
+
 static void
 zoom_level_changed_callback (NautilusView *view,
                              NautilusNavigationWindow *window)
@@ -728,10 +842,11 @@ static void
 real_set_content_view_widget (NautilusWindow *nautilus_window,
 			      NautilusView *new_view)
 {
+	
 	NautilusNavigationWindow *window;
-	
+
 	window = NAUTILUS_NAVIGATION_WINDOW (nautilus_window);
-	
+
 	disconnect_view (window, nautilus_window->content_view);
 
 	EEL_CALL_PARENT (NAUTILUS_WINDOW_CLASS, 
@@ -778,6 +893,10 @@ nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWind
 {
 	if (!nautilus_navigation_window_location_bar_showing (window)) {
 		nautilus_navigation_window_show_location_bar (window, FALSE);
+		window->details->temporary_location_bar = TRUE;
+	}
+	if (nautilus_navigation_window_path_bar_showing (window)) {
+		nautilus_navigation_window_hide_path_bar (window);
 		window->details->temporary_navigation_bar = TRUE;
 	}
 	nautilus_navigation_bar_activate 
@@ -866,7 +985,7 @@ add_sidebar_panels (NautilusNavigationWindow *window)
 void 
 nautilus_navigation_window_hide_location_bar (NautilusNavigationWindow *window, gboolean save_preference)
 {
-	window->details->temporary_navigation_bar = FALSE;
+	window->details->temporary_location_bar = FALSE;
 	gtk_widget_hide (window->details->location_bar);
 	nautilus_navigation_window_update_show_hide_menu_items (window);
 	if (save_preference &&
@@ -896,6 +1015,31 @@ nautilus_navigation_window_location_bar_showing (NautilusNavigationWindow *windo
 	return TRUE;
 }
 
+void 
+nautilus_navigation_window_hide_path_bar (NautilusNavigationWindow *window)
+{
+	window->details->temporary_navigation_bar = FALSE;
+	gtk_widget_hide (window->path_bar);
+	gtk_widget_show (window->navigation_bar);
+}
+
+void 
+nautilus_navigation_window_show_path_bar (NautilusNavigationWindow *window)
+{
+	gtk_widget_show (window->path_bar);
+	gtk_widget_hide (window->navigation_bar);
+}
+
+gboolean
+nautilus_navigation_window_path_bar_showing (NautilusNavigationWindow *window)
+{
+	if (window->path_bar != NULL) {
+		return GTK_WIDGET_VISIBLE (window->path_bar);
+	}
+	/* If we're not visible yet we haven't changed visibility, so its TRUE */
+	return TRUE;
+}
+
 gboolean
 nautilus_navigation_window_toolbar_showing (NautilusNavigationWindow *window)
 {
@@ -940,6 +1084,29 @@ nautilus_navigation_window_status_bar_showing (NautilusNavigationWindow *window)
 	return TRUE;
 }
 
+
+void 
+nautilus_navigation_window_hide_toolbar (NautilusNavigationWindow *window)
+{
+	gtk_widget_hide (window->details->toolbar);
+	nautilus_navigation_window_update_show_hide_menu_items (window);
+	if (eel_preferences_key_is_writable (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR) &&
+	    eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR)) {
+		eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR, FALSE);
+	}
+}
+
+void 
+nautilus_navigation_window_show_toolbar (NautilusNavigationWindow *window)
+{
+	gtk_widget_show (window->details->toolbar);
+	nautilus_navigation_window_update_show_hide_menu_items (window);
+	if (eel_preferences_key_is_writable (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR) &&
+	    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR)) {
+		eel_preferences_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR, TRUE);
+	}
+}
+
 void 
 nautilus_navigation_window_hide_sidebar (NautilusNavigationWindow *window)
 {
@@ -1007,6 +1174,8 @@ nautilus_navigation_window_get_base_page_index (NautilusNavigationWindow *window
 	return forward_count;
 }
 
+
+
 /**
  * nautilus_navigation_window_show:
  * @widget:	GtkWidget
@@ -1024,12 +1193,25 @@ nautilus_navigation_window_show (GtkWidget *widget)
 	/* Initially show or hide views based on preferences; once the window is displayed
 	 * these can be controlled on a per-window basis from View menu items. 
 	 */
+
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR)) {
+		nautilus_navigation_window_show_toolbar (window);
+	} else {
+		nautilus_navigation_window_hide_toolbar (window);
+	}
+
 	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR)) {
 		nautilus_navigation_window_show_location_bar (window, FALSE);
 	} else {
 		nautilus_navigation_window_hide_location_bar (window, FALSE);
 	}
 
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY)) {
+		nautilus_navigation_window_hide_path_bar (window);
+	} else {
+		nautilus_navigation_window_show_path_bar (window);
+	}
+	
 	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR)) {
 		nautilus_navigation_window_show_sidebar (window);
 	} else {
@@ -1090,7 +1272,8 @@ static void
 nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 {
 	NAUTILUS_WINDOW_CLASS (class)->window_type = NAUTILUS_WINDOW_NAVIGATION;
-
+	NAUTILUS_WINDOW_CLASS (class)->bookmarks_placeholder = MENU_PATH_BOOKMARKS_PLACEHOLDER;
+	
 	G_OBJECT_CLASS (class)->finalize = nautilus_navigation_window_finalize;
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_navigation_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_navigation_window_show;
@@ -1100,6 +1283,7 @@ nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 	NAUTILUS_WINDOW_CLASS (class)->set_throbber_active = real_set_throbber_active;
 	NAUTILUS_WINDOW_CLASS (class)->prompt_for_location = real_prompt_for_location;
 	NAUTILUS_WINDOW_CLASS (class)->set_title = real_set_title;
+	NAUTILUS_WINDOW_CLASS (class)->get_icon_name = real_get_icon_name;
 	NAUTILUS_WINDOW_CLASS(class)->get_default_size = real_get_default_size;
 	NAUTILUS_WINDOW_CLASS (class)->close = real_window_close;
 }
