@@ -30,6 +30,7 @@
 #include <config.h>
 #include "nautilus-spatial-window.h"
 #include "nautilus-window-private.h"
+#include "nautilus-window-bookmarks.h"
 
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
@@ -48,6 +49,7 @@
 #include <eel/eel-string.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtkdnd.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkmenuitem.h>
@@ -63,6 +65,7 @@
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
@@ -79,6 +82,12 @@
 
 #define MAX_TITLE_LENGTH 180
 
+#define SPATIAL_ACTION_PLACES               "Places"
+#define SPATIAL_ACTION_GO_TO_LOCATION       "Go to Location"
+#define SPATIAL_ACTION_CLOSE_PARENT_FOLDERS "Close Parent Folders"
+#define SPATIAL_ACTION_CLOSE_ALL_FOLDERS    "Close All Folders"
+#define MENU_PATH_SPATIAL_BOOKMARKS_PLACEHOLDER	"/MenuBar/Other Menus/Places/Bookmarks Placeholder"
+
 struct _NautilusSpatialWindowDetails {
         GtkActionGroup *spatial_action_group; /* owned by ui_manager */
 	char *last_geometry;	
@@ -87,10 +96,17 @@ struct _NautilusSpatialWindowDetails {
 	GtkWidget *content_box;
 	GtkWidget *location_button;
 	GtkWidget *location_label;
+	GtkWidget *location_icon;
 
 	GnomeVFSURI *location;
 };
 
+static const GtkTargetEntry location_button_drag_types[] = {
+	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
+	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
+	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
+};
+
 GNOME_CLASS_BOILERPLATE (NautilusSpatialWindow, nautilus_spatial_window,
 			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
 
@@ -170,28 +186,6 @@ nautilus_spatial_window_unrealize (GtkWidget *widget)
 	}
 }
 
-
-static void
-nautilus_spatial_window_realize (GtkWidget *widget)
-{
-	NautilusSpatialWindow *window;
-
-	window = NAUTILUS_SPATIAL_WINDOW (widget);
-
-	GTK_WIDGET_CLASS (parent_class)->realize (widget);
-
-	if (window->loading) {
-		GdkCursor *cursor;
-
-		cursor = gdk_cursor_new (GDK_WATCH);
-		gdk_window_set_cursor (widget->window, cursor);
-		gdk_cursor_unref (cursor);
-	} else {
-		gdk_window_set_cursor (widget->window, NULL);
-	}
-}
-
-
 static void
 nautilus_spatial_window_destroy (GtkObject *object)
 {
@@ -319,6 +313,12 @@ real_prompt_for_location (NautilusWindow *window)
 	gtk_widget_show (dialog);
 }
 
+static char *
+real_get_icon_name (NautilusWindow *window)
+{
+	return nautilus_icon_factory_get_icon_for_file (window->details->viewed_file, FALSE);
+}
+
 static void
 real_set_title (NautilusWindow *window, const char *title)
 {
@@ -376,28 +376,22 @@ real_get_default_size (NautilusWindow *window,
 static void
 real_set_throbber_active (NautilusWindow *window, gboolean active)
 {
-	NautilusSpatialWindow *spatial;
-
-	spatial = NAUTILUS_SPATIAL_WINDOW (window);
-	spatial->loading = active;
-	
-	if (!GTK_WIDGET_REALIZED (GTK_WIDGET (window)))
-		return;
-
-	if (active) {
-		GdkCursor *cursor;
-
-		cursor = gdk_cursor_new (GDK_WATCH);
-		gdk_window_set_cursor (GTK_WIDGET (window)->window,
-				       cursor);
-		gdk_cursor_unref (cursor);
-		
-	} else {
-		gdk_window_set_cursor (GTK_WIDGET (window)->window,
-				       NULL);
-	}
 }
 
+static void
+real_set_allow_up (NautilusWindow *window, gboolean allow)
+{
+	NautilusSpatialWindow *spatial;
+	GtkAction *action;
+
+	spatial = NAUTILUS_SPATIAL_WINDOW (window);
+
+	action = gtk_action_group_get_action (spatial->details->spatial_action_group,
+					      SPATIAL_ACTION_CLOSE_PARENT_FOLDERS);
+	gtk_action_set_sensitive (action, allow);
+
+	NAUTILUS_WINDOW_CLASS (parent_class)->set_allow_up (window, allow);
+}
 
 static void
 location_menu_item_activated_callback (GtkWidget *menu_item,
@@ -442,6 +436,30 @@ location_menu_item_activated_callback (GtkWidget *menu_item,
 	
 }
 
+static void
+got_file_info_for_location_menu_callback (NautilusFile *file,
+					  gpointer callback_data)
+{	
+	GtkWidget *icon;
+	GtkWidget *menu_item = callback_data;
+	char *icon_name;
+
+	g_return_if_fail (NAUTILUS_IS_FILE (file));
+
+	icon_name = nautilus_icon_factory_get_icon_for_file (file, FALSE);
+	if (icon_name) {
+		icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	} else {
+		icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+	}
+	
+	if (icon) {
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
+	}
+	g_object_unref (file);
+	g_object_unref (menu_item);
+}
+
 static void
 menu_deactivate_callback (GtkWidget *menu,
 			  gpointer   data)
@@ -479,10 +497,23 @@ menu_popup_pos (GtkMenu   *menu,
 	*push_in = TRUE;
 }
 
+static gboolean
+location_button_pressed_callback (GtkWidget      *widget,
+				  GdkEventButton *event,
+				  NautilusWindow *window)
+{
+	if (event->button == 3 &&
+	    window->content_view != NULL) {
+		nautilus_view_pop_up_location_context_menu (window->content_view, event);
+	}
+
+	return FALSE;
+}
+
 static void
 location_button_clicked_callback (GtkWidget *widget, NautilusSpatialWindow *window)
 {
-	GtkWidget *popup, *menu_item, *first_item;
+	GtkWidget *popup, *menu_item, *first_item = NULL;
 	GnomeVFSURI *uri;
 	GnomeVFSURI *child_uri;
 	char *name;
@@ -496,17 +527,25 @@ location_button_clicked_callback (GtkWidget *widget, NautilusSpatialWindow *wind
 	uri = gnome_vfs_uri_ref (window->details->location);
 	child_uri = NULL;
 	while (uri != NULL) {
+		NautilusFile *file;
+		char *uri_string;
+
 		name = nautilus_get_uri_shortname_for_display (uri);
 		menu_item = gtk_image_menu_item_new_with_label (name);
+		g_free (name);
 		if (first_item == NULL) {
-			GtkWidget *open_icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
-
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), open_icon);
-
 			first_item = menu_item;
 		}
 		
-		g_free (name);
+		uri_string = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+		file = nautilus_file_get (uri_string);
+		g_object_ref (menu_item);
+		nautilus_file_call_when_ready (file,
+					       NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY,
+					       got_file_info_for_location_menu_callback,
+					       menu_item);
+		g_free (uri_string);		
+
 		gtk_widget_show (menu_item);
 		g_signal_connect (menu_item, "activate",
 				  G_CALLBACK (location_menu_item_activated_callback),
@@ -536,6 +575,80 @@ location_button_clicked_callback (GtkWidget *widget, NautilusSpatialWindow *wind
  	gtk_object_sink (GTK_OBJECT (popup));
 }
 
+static int
+get_dnd_icon_size (NautilusSpatialWindow *window)
+{
+	NautilusWindow *parent;
+	NautilusZoomLevel zoom_level;
+
+	parent = NAUTILUS_WINDOW(window);
+
+	if (parent->content_view == NULL) {
+		return NAUTILUS_ICON_SIZE_STANDARD;
+	} else {
+		zoom_level = nautilus_view_get_zoom_level (parent->content_view);
+		return nautilus_get_icon_size_for_zoom_level (zoom_level);
+	}
+	
+}
+
+static void
+location_button_drag_begin_callback (GtkWidget             *widget,
+				     GdkDragContext        *context,
+				     NautilusSpatialWindow *window)
+{
+	GdkPixbuf *pixbuf;
+
+	pixbuf = nautilus_icon_factory_get_pixbuf_for_file (NAUTILUS_WINDOW (window)->details->viewed_file,
+							    "open", get_dnd_icon_size (window));
+
+	gtk_drag_set_icon_pixbuf (context, pixbuf, 0, 0);
+
+	g_object_unref (pixbuf);
+}
+
+/* build GNOME icon list, which only contains the window's URI.
+ * If we just used URIs, moving the folder to trash
+ * wouldn't work */
+static void
+get_data_binder (NautilusDragEachSelectedItemDataGet iteratee,
+		 gpointer                            iterator_context,
+		 gpointer                            data)
+{
+	NautilusSpatialWindow *window;
+	int icon_size;
+	char *uri;
+
+	g_assert (NAUTILUS_IS_SPATIAL_WINDOW (iterator_context));
+	window = NAUTILUS_SPATIAL_WINDOW (iterator_context);
+
+	uri = gnome_vfs_uri_to_string (window->details->location,
+				       GNOME_VFS_URI_HIDE_NONE);
+
+	icon_size = get_dnd_icon_size (window);
+
+	iteratee (uri,
+		  0,
+		  0,
+		  icon_size,
+		  icon_size,
+		  data);
+
+	g_free (uri);
+}
+
+static void
+location_button_drag_data_get_callback (GtkWidget             *widget,
+					GdkDragContext        *context,
+					GtkSelectionData      *selection_data,
+					guint                  info,
+					guint                  time,
+					NautilusSpatialWindow *window)
+{
+	nautilus_drag_drag_data_get (widget, context, selection_data,
+				     info, time, window, get_data_binder); 
+}
+
 void
 nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
 					      const char            *location)
@@ -548,11 +661,32 @@ nautilus_spatial_window_set_location_button  (NautilusSpatialWindow *window,
 		uri = gnome_vfs_uri_new (location);
 	}
 	if (uri != NULL) {
+		NautilusFile *file;
+		GnomeVFSResult vfs_result_code;
+		
 		name = nautilus_get_uri_shortname_for_display (uri);
 		gtk_label_set_label (GTK_LABEL (window->details->location_label),
 				     name);
 		g_free (name);
 		gtk_widget_set_sensitive (window->details->location_button, TRUE);
+
+		file = nautilus_file_get (location);
+		vfs_result_code = nautilus_file_get_file_info_result (file);
+		if (vfs_result_code == GNOME_VFS_OK) {
+			char *icon_name;
+
+			icon_name = nautilus_icon_factory_get_icon_for_file (file, FALSE);		
+			if (icon_name) {
+				gtk_image_set_from_icon_name (GTK_IMAGE (window->details->location_icon), 
+							      icon_name, GTK_ICON_SIZE_MENU);
+				g_free (icon_name);
+			} else {
+				gtk_image_set_from_stock (GTK_IMAGE (window->details->location_icon),
+							  GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+			}
+		}
+		g_object_unref (file);
+
 	} else {
 		gtk_label_set_label (GTK_LABEL (window->details->location_label),
 				     "");
@@ -576,17 +710,37 @@ action_go_to_location_callback (GtkAction *action,
 	nautilus_window_prompt_for_location (window);
 }			   
 
-static GtkActionEntry spatial_entries[] = {
-  { "Places", NULL, N_("_Places") },               /* name, stock id, label */
-  { "Go to Location", NULL, N_("Open _Location..."), /* name, stock id, label */
+static void
+action_add_bookmark_callback (GtkAction *action,
+			      gpointer user_data)
+{
+        nautilus_window_add_bookmark_for_current_location (NAUTILUS_WINDOW (user_data));
+}
+
+static void
+action_edit_bookmarks_callback (GtkAction *action, 
+				gpointer user_data)
+{
+        nautilus_window_edit_bookmarks (NAUTILUS_WINDOW (user_data));
+}
+
+static const GtkActionEntry spatial_entries[] = {
+  { SPATIAL_ACTION_PLACES, NULL, N_("_Places") },               /* name, stock id, label */
+  { SPATIAL_ACTION_GO_TO_LOCATION, NULL, N_("Open _Location..."), /* name, stock id, label */
     "<control>L", N_("Specify a location to open"),
     G_CALLBACK (action_go_to_location_callback) },
-  { "Close Parent Folders", NULL, N_("Close P_arent Folders"), /* name, stock id, label */
+  { SPATIAL_ACTION_CLOSE_PARENT_FOLDERS, NULL, N_("Close P_arent Folders"), /* name, stock id, label */
     "<control><shift>W", N_("Close this folder's parents"),
     G_CALLBACK (action_close_parent_folders_callback) },
-  { "Close All Folders", NULL, N_("Clos_e All Folders"), /* name, stock id, label */
+  { SPATIAL_ACTION_CLOSE_ALL_FOLDERS, NULL, N_("Clos_e All Folders"), /* name, stock id, label */
     "<control>Q", N_("Close all folder windows"),
     G_CALLBACK (action_close_all_folders_callback) },
+  { "Add Bookmark", GTK_STOCK_ADD, N_("_Add Bookmark"), /* name, stock id, label */
+    "<control>d", N_("Add a bookmark for the current location to this menu"),
+    G_CALLBACK (action_add_bookmark_callback) },
+  { "Edit Bookmarks", NULL, N_("_Edit Bookmarks"), /* name, stock id, label */
+    "<control>b", N_("Display a window that allows editing the bookmarks in this menu"),
+    G_CALLBACK (action_edit_bookmarks_callback) },
 };
 
 static void
@@ -597,8 +751,8 @@ nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 	GtkWidget *hbox;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
+	GtkTargetList *targets;
 	const char *ui;
-	GtkWidget *folder_icon;
 	
 	window->details = g_new0 (NautilusSpatialWindowDetails, 1);
 	window->affect_spatial_window_on_next_location_change = TRUE;
@@ -607,13 +761,17 @@ nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 		gtk_hbox_new (FALSE, 0);
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
 			  window->details->content_box,
-			  /* X direction */       /* Y direction */
-			  0, 1,                   1, 4,
-			  GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
-			  0,                      0);
+			  /* X direction */                   /* Y direction */
+			  0, 1,                               1, 4,
+			  GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_EXPAND | GTK_FILL | GTK_SHRINK,
+			  0,                                  0);
 	gtk_widget_show (window->details->content_box);
 
 	window->details->location_button = gtk_button_new ();
+	g_signal_connect (window->details->location_button,
+			  "button-press-event",
+			  G_CALLBACK (location_button_pressed_callback),
+			  window);
 	gtk_button_set_relief (GTK_BUTTON (window->details->location_button),
 			       GTK_RELIEF_NORMAL);
 	rc_style = gtk_widget_get_modifier_style (window->details->location_button);
@@ -623,15 +781,14 @@ nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 				 rc_style);
 
 	gtk_widget_show (window->details->location_button);
-
 	hbox = gtk_hbox_new (FALSE, 3);
 	gtk_container_add (GTK_CONTAINER (window->details->location_button), 
 			   hbox);
 	gtk_widget_show (hbox);
 
-	folder_icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
-	gtk_box_pack_start (GTK_BOX (hbox), folder_icon, FALSE, FALSE, 0);
-	gtk_widget_show (folder_icon);
+	window->details->location_icon = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+	gtk_box_pack_start (GTK_BOX (hbox), window->details->location_icon, FALSE, FALSE, 0);
+	gtk_widget_show (window->details->location_icon);
 	
 	window->details->location_label = gtk_label_new ("");
 	gtk_box_pack_start (GTK_BOX (hbox), window->details->location_label,
@@ -642,7 +799,22 @@ nautilus_spatial_window_instance_init (NautilusSpatialWindow *window)
 	gtk_box_pack_start (GTK_BOX (hbox), arrow, FALSE, FALSE, 0);
 	gtk_widget_show (arrow);
 
-	
+	gtk_drag_source_set (window->details->location_button,
+			     GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, location_button_drag_types,
+			     G_N_ELEMENTS (location_button_drag_types),
+			     GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK);
+	g_signal_connect (window->details->location_button,
+			  "drag_begin",
+			  G_CALLBACK (location_button_drag_begin_callback),
+			  window);
+	g_signal_connect (window->details->location_button,
+			  "drag_data_get",
+			  G_CALLBACK (location_button_drag_data_get_callback),
+			  window);
+
+	targets = gtk_drag_source_get_target_list (window->details->location_button);
+	gtk_target_list_add_text_targets (targets, NAUTILUS_ICON_DND_TEXT);
+
 	gtk_widget_set_sensitive (window->details->location_button, FALSE);
 	g_signal_connect (window->details->location_button, 
 			  "clicked", 
@@ -677,16 +849,18 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 	GtkBindingSet *binding_set;
 	
 	NAUTILUS_WINDOW_CLASS (class)->window_type = NAUTILUS_WINDOW_SPATIAL;
+	NAUTILUS_WINDOW_CLASS (class)->bookmarks_placeholder = MENU_PATH_SPATIAL_BOOKMARKS_PLACEHOLDER;
 
 	G_OBJECT_CLASS (class)->finalize = nautilus_spatial_window_finalize;
 	GTK_OBJECT_CLASS (class)->destroy = nautilus_spatial_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_spatial_window_show;
 	GTK_WIDGET_CLASS (class)->configure_event = nautilus_spatial_window_configure_event;
 	GTK_WIDGET_CLASS (class)->unrealize = nautilus_spatial_window_unrealize;
-	GTK_WIDGET_CLASS (class)->realize = nautilus_spatial_window_realize;
 
 	NAUTILUS_WINDOW_CLASS (class)->prompt_for_location = 
 		real_prompt_for_location;
+	NAUTILUS_WINDOW_CLASS (class)->get_icon_name =
+		real_get_icon_name;
 	NAUTILUS_WINDOW_CLASS (class)->set_title = 
 		real_set_title;
 	NAUTILUS_WINDOW_CLASS (class)->set_content_view_widget = 
@@ -697,7 +871,8 @@ nautilus_spatial_window_class_init (NautilusSpatialWindowClass *class)
 
 	NAUTILUS_WINDOW_CLASS(class)->set_throbber_active =
 		real_set_throbber_active;
-
+	NAUTILUS_WINDOW_CLASS(class)->set_allow_up =
+		real_set_allow_up;
 
 	binding_set = gtk_binding_set_by_class (class);
 	gtk_binding_entry_add_signal (binding_set, GDK_BackSpace, GDK_SHIFT_MASK,
